// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  TU Dresden                                                            ==
// ==                                                                        ==
// ==  Institut fr Wissenschaftliches Rechnen                               ==
// ==  Zellescher Weg 12-14                                                  ==
// ==  01069 Dresden                                                         ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  https://gforge.zih.tu-dresden.de/projects/amdis/                      ==
// ==                                                                        ==
// ============================================================================

/** \file PartitionElementData.h */

#ifndef AMDIS_PARTITIONELEMENTDATA_H
#define AMDIS_PARTITIONELEMENTDATA_H

#include "Element.h"
#include "ElementData.h"
#include "FixVec.h"

namespace AMDiS {

  const int PARTITION_ED = 7;

  enum PartitionStatus {
    UNDEFINED = -1,
    IN = 0,
    OVERLAP = 1,
    OUT = 2
  };

  class PartitionElementData : public ElementData
  {
  public:
    inline bool isOfType(int typeID) const {
      if (typeID == PARTITION_ED) 
			return true;
      return false;
    }

    class Creator : public CreatorInterface<ElementData>
    {
    public:
      ElementData* create() {
	return new PartitionElementData;
      }
    };

    PartitionElementData(ElementData *decorated = NULL)
      : ElementData(decorated),
	status(UNDEFINED),
	level(0)
    {}

    ~PartitionElementData()
    {}

    bool refineElementData(Element* parent, 
			   Element* child1,
			   Element* child2,
			   int elType)
    {
	ElementData::refineElementData(parent, child1, child2, elType);
	
      PartitionElementData *child1Data = new PartitionElementData(child1->getElementData());
      PartitionElementData *child2Data = new PartitionElementData(child2->getElementData());
      child1Data->setPartitionStatus(status);
      child2Data->setPartitionStatus(status);
      child1Data->setLevel(level + 1);
      child2Data->setLevel(level + 1);
      child1->setElementData(child1Data);
      child2->setElementData(child2Data);
      return false;
    };

    ElementData *clone() const { 
      PartitionElementData *newObj = new PartitionElementData;
      newObj->decorated_ = ElementData::clone();
      return newObj; 
    };

    inline std::string getTypeName() const { 
      return "PartitionElementData"; 
    };

    inline const int getTypeID() const { 
      return PARTITION_ED; 
    };

    void serialize(std::ostream& out) 
    {
      ElementData::serialize(out);
      out.write(reinterpret_cast<const char*>(&status), sizeof(PartitionStatus));
      out.write(reinterpret_cast<const char*>(&level), sizeof(int));
    };

    void deserialize(std::istream& in) 
    {
      ElementData::deserialize(in);
      in.read(reinterpret_cast<char*>(&status), sizeof(PartitionStatus));
      in.read(reinterpret_cast<char*>(&level), sizeof(int));
    };

    inline void setPartitionStatus(PartitionStatus status_) { 
      status = status_; 
    };

    inline PartitionStatus getPartitionStatus() { 
      return status; 
    };

    inline void setLevel(int level_) {
      level = level_;
    };

    inline int getLevel() { 
      return level; 
    };

    void descend(Element *element) 
    {
      if (!element->isLeaf()) {
	Element *child0 = element->getChild(0);
	Element *child1 = element->getChild(1);

	// get partition data
	PartitionElementData *child0Data = dynamic_cast<PartitionElementData*>
	  (child0->getElementData(PARTITION_ED));
	PartitionElementData *child1Data = dynamic_cast<PartitionElementData*>
	  (child1->getElementData(PARTITION_ED));
      
	TEST_EXIT(child0Data && child1Data)("no partition data\n");

	child0Data->setPartitionStatus(this->getPartitionStatus());
	child1Data->setPartitionStatus(this->getPartitionStatus());

	child0Data->descend(child0);
	child1Data->descend(child1);
      }    
    };

  protected:
    PartitionStatus status;

    int level;
  };

}

#endif
