// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  TU Dresden                                                            ==
// ==                                                                        ==
// ==  Institut fr Wissenschaftliches Rechnen                               ==
// ==  Zellescher Weg 12-14                                                  ==
// ==  01069 Dresden                                                         ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  https://gforge.zih.tu-dresden.de/projects/amdis/                      ==
// ==                                                                        ==
// ============================================================================

/** \file SurfaceRegion_ED.h */

#ifndef AMDIS_SURFACEREGION_H
#define AMDIS_SURFACEREGION_H

#include "ElementData.h"
#include "FixVec.h"

namespace AMDiS {

  class SurfaceRegion_ED : public ElementData
  {
  public:
    inline bool isOfType(int typeID) const {
      if (typeID == SURFACE_REGION) 
	return true;
      return false;
    }

    class Creator : public CreatorInterface<ElementData>
    {
    public:
      ElementData* create() 
      {
	return new SurfaceRegion_ED;
      }
    };

    SurfaceRegion_ED(ElementData *decorated = NULL)
      : ElementData(decorated),
	side_(-1),
	region_(-1)
    {}

    bool refineElementData(Element* parent, 
			   Element* child1,
			   Element* child2,
			   int elType)
    {
      ElementData::refineElementData(parent, child1, child2, elType);

      int sideOfChild;
      SurfaceRegion_ED *surfaceRegion;

      sideOfChild = parent->getSideOfChild(0, side_, elType);
      if (sideOfChild >= 0) {
	surfaceRegion = new SurfaceRegion_ED(child1->getElementData());
	surfaceRegion->setSide(sideOfChild);
	surfaceRegion->setRegion(region_);
	child1->setElementData(surfaceRegion);
      }    

      sideOfChild = parent->getSideOfChild(1, side_, elType);
      if (sideOfChild >= 0) {
	surfaceRegion = new SurfaceRegion_ED(child2->getElementData());
	surfaceRegion->side_ = sideOfChild;
	surfaceRegion->region_ = region_;
	child2->setElementData(surfaceRegion);
      }

      return false;
    }

    ElementData *clone() const 
    { 
      SurfaceRegion_ED *newObj = new SurfaceRegion_ED;
      newObj->side_ = side_;
      newObj->region_ = region_;
      newObj->decorated_ = ElementData::clone();
      return newObj; 
    }

    inline std::string getTypeName() const 
    { 
      return "SurfaceRegion_ED"; 
    }

    inline const int getTypeID() const 
    { 
      return SURFACE_REGION; 
    }

    void serialize(std::ostream& out) 
    {
      ElementData::serialize(out);
      out.write(reinterpret_cast<const char*>(&side_), sizeof(int));
      out.write(reinterpret_cast<const char*>(&region_), sizeof(int));
    }

    void deserialize(std::istream& in) 
    {
      ElementData::deserialize(in);
      in.read(reinterpret_cast<char*>(&side_), sizeof(int));
      in.read(reinterpret_cast<char*>(&region_), sizeof(int));
    }

    inline void setSide(int side) 
    { 
      side_ = side; 
    }

    inline int getSide() const 
    { 
      return side_; 
    }

    inline void setRegion(int region) 
    { 
      region_ = region; 
    }

    inline int getRegion() const 
    { 
      return region_; 
    }

  protected:
    int side_;
    int region_;
  };

}

#endif
